<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://wildrobot.app/logistra
 * @since      1.0.0
 *
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/admin
 * @author     Robertosnap <robertosnap+spam@pm.me>
 */
class Logistra_Robots_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */

    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->setting_name = 'logistra_robots';
        $this->securityNounce = 'randomTextForLogistraIntegration';

        add_filter('woocommerce_settings_tabs_array', array($this, 'addSettingsTab'), 90);
        add_action('woocommerce_settings_tabs_' . $this->setting_name . '_tab', array($this, 'settingsTab'));
        add_action('woocommerce_admin_order_actions_end', array($this, 'sendOrderAction'));
        add_action('woocommerce_shipping_zone_method_deleted', array($this, 'deleteTransportRelation'), 10, 3);

        // Checkout field for service partner
        add_action('woocommerce_admin_order_data_after_shipping_address', array($this, 'add_service_partner_picker_field_to_checkout'), 10, 1);

        // Filter for static weight
        add_filter('logistra_robots_set_order_weight', array($this, 'set_static_weight_on_transports'), 10, 2);

        // Add tracking URL to order emails
        add_filter('woocommerce_email_order_meta_fields', array($this, 'display_tracking_in_email'), 10, 3);

        // Add box on edit order screen
        add_action('add_meta_boxes_shop_order', array($this, 'robots_logistra_box_send'));
        add_action('add_meta_boxes_shop_order', array($this, 'robots_logistra_box_return'));

        // Add SubMenu 
        add_action('admin_menu', array($this, 'robots_logistra_submenu_page'), 99);

        // Send order when proccessed
        add_action('woocommerce_order_status_completed', array($this, 'send_order'), 10);

        // Adding to admin order list bulk dropdown a custom action 'send_order_transport'
        add_filter('bulk_actions-edit-shop_order', array($this, 'bulk_actions_send_order_transport'), 20, 1);
        // Action for bulk
        add_filter('handle_bulk_actions-edit-shop_order', array($this, 'bulk_action_send_order_transport'), 10, 3);
        // The results notice from bulk action on orders
        add_action('admin_notices', array($this, 'bulk_action_send_order_transport_admin_notice'));
    }

    public function bulk_action_send_order_transport_admin_notice()
    {
        if (empty($_REQUEST['orders_sent_transport'])) return; // Exit

        $count = intval($_REQUEST['processed_count']);
        $count_error = intval($_REQUEST['errors_count']);

        printf('<div id="message" class="updated fade"><p>' .
            _n(
                'Sendte %s / %s ordre for transport.',
                'Sendte %s / %s ordre for transport. Sjekk Woocommerce -> Status -> Logs -> logistra-robots-bulk-*.log for å se årsak.',
                $count_error + 1,
                'orders_sent_transport'
            ) . '</p></div>', $count, $count + $count_error);
    }

    public function bulk_action_send_order_transport($redirect_to, $action, $post_ids)
    {
        if ($action !== 'send_order_transport')
            return $redirect_to; // Exit

        $logger = new WC_Logger();
        $context = array('source' => 'logistra-robots-bulk');
        $logger->info("_________Startet bulk send ordre for transport______________", $context);
        $logger->info(wc_print_r($post_ids, true), $context);

        $processed_ids = array();
        $error_ids = array();

        foreach ($post_ids as $orderId) {

            require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-helpers.php';
            $required_options_check = Logistra_Robots_Helpers::check_required_options($orderId);
            if ($required_options_check['error']) {
                $logger->error("check_required_options", $context);
                $logger->info("orderId =>" . $orderId . " with errror " . $required_options_check['message'], $context);
                $error_ids[] = $orderId;
                continue;
            }

            require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-order.php';

            $logistra_order = new Logistra_Robots_Order($orderId);
            $consignmentsArray = $logistra_order->order_to_consignments(array());
            if (empty($consignmentsArray)) {
                $logger->error("Send order to transport error when creating consignments.", $context);
                $logger->info("orderId =>" . $orderId . " with errror " . "There was no shipping methods on the order.", $context);
                $error_ids[] = $orderId;
                continue;
            }

            foreach ($consignmentsArray as $consignments) {
                $createConsignmentResponse = Logistra_Robots_Backend::create_consignments($consignments);
                $logistra_order->handle_created_consignments($createConsignmentResponse);
            }


            $logger->info("send order error " . wc_print_r($logistra_order->error, true), $context);
            $logger->info("send order message : " . wc_print_r($logistra_order->messages, true), $context);

            if ($logistra_order->error == false) {
                $processed_ids[] = $orderId;
            } else {
                $logger->error("Send order to transport error in response from Logistra.", $context);
                $logger->info("orderId =>" . $orderId . " with errror " . $logistra_order->messages[0], $context);
                $error_ids[] = $orderId;
                continue;
            }
        }
        return $redirect_to = add_query_arg(array(
            'orders_sent_transport' => 1,
            'processed_count' => count($processed_ids),
            'errors_count' => count($error_ids),
            'processed_ids' => implode(',', $processed_ids),
        ), $redirect_to);
    }

    public function bulk_actions_send_order_transport($actions)
    {
        $actions['send_order_transport'] = __('Send ordre med transport', 'logistra-robots');
        return $actions;
    }

    public function send_order($order_id)
    {
        $send_order_on_complete_order = get_option('logistra-robots-setting-send-consignment-on-complete-order', null) === 'yes';
        if ($send_order_on_complete_order) {
            try {
                require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-order.php';

                $logistra_order = new Logistra_Robots_Order($order_id);
                $consignmentsArray = $logistra_order->order_to_consignments(array());

                foreach ($consignmentsArray as $consignments) {
                    $createConsignmentResponse = Logistra_Robots_Backend::create_consignments($consignments);
                    $logistra_order->handle_created_consignments($createConsignmentResponse);
                }
            } catch (Exception $e) {
                echo 'Caught exception: ',  $e->getMessage(), "\n";
            }
        }
    }

    public function robots_logistra_submenu_page()
    {
        add_submenu_page('woocommerce', 'Transport', 'Transport', 'manage_options', admin_url("admin.php?page=wc-settings&tab=logistra_robots_tab"));
    }

    public function robots_logistra_box_send()
    {
        add_meta_box(
            'logistra_robots_send',
            __('Frakt', 'logistra-robots'),
            array($this, 'robots_logistra_box_content_send'),
            'shop_order',
            'side',
            'high'
        );
    }

    public function robots_logistra_box_return()
    {
        add_meta_box(
            'logistra_robots_return',
            __('Retur', 'logistra-robots'),
            array($this, 'robots_logistra_box_content_return'),
            'shop_order',
            'side',
            'high'
        );
    }

    public function robots_logistra_box_content_send()
    {
        $order_id = intval($_GET['post']);

        // To show if order has been sent
        $sent = (get_post_meta($order_id, 'logistra-robots-sent', true) == 'yes') ? "true" : "false";

        echo "<div id='logistra-robots-send-order-content-box' data-id=" . $order_id . " data-sent=" . $sent . " ></div>";
    }

    public function robots_logistra_box_content_return()
    {
        $order_id = intval($_GET['post']);

        // To show if order has been sent
        $sent = (get_post_meta($order_id, 'logistra-robots-sent', true) == 'yes') ? "true" : "false";

        echo "<div id='logistra-robots-return-order-content-box' data-id=" . $order_id . " data-sent=" . $sent . "></div>";
    }

    public function display_tracking_in_email($fields, $sent_to_admin, $order)
    {

        $tracking_url = get_post_meta($order->get_id(), 'logistra-robots-tracking-url', true);

        if(get_option("logistra-robots-freight-track-url-email", null) !== 'no' && strlen($tracking_url) > 0){
            array_push($fields, array(
                'label' => __('Sporing', 'logistra-robots'),
                'value' => __(wp_sprintf('<a href="%s">Klikk her</a>', $tracking_url)),
            ));
        }
      

        return $fields;
    }

    public function set_static_weight_on_transports($weight, $order_id)
    {
        if (get_option('logistra-robots-static-weight-on-orders', 'no') == 'yes') {
            $weight = get_option('logistra-robots-static-weight-in-kg', 0);
        }
        return $weight;
    }

    public function add_service_partner_picker_field_to_checkout($order)
    {
        echo '<p><strong>' . __('Utleveringssted') . ':</strong> ' . get_post_meta($order->get_id(), '_shipping_service_partner', true) . '</p>';
    }

    function sendOrderAction($order)
    {
        // We need to attach some data to the DOM element to instantiate the VUE components correctly and wihtout extra backend calls
        // To show if order has been sent
        $sent = (get_post_meta($order->get_id(), 'logistra-robots-sent', true) == 'yes') ? "true" : "false";
        // To distiguish each button /vue instance
        $order_id = $order->get_id();
        echo "<div class='logistra-robots-send-order' data-id=" . $order_id . " data-sent=" . $sent . " ></div>";
    }


    public function addSettingsTab($settings_tabs)
    {
        $tab_name = $this->setting_name . '_tab';
        $settings_tabs[$tab_name] = __('Logistra', 'logistra-robots');
        return $settings_tabs;
    }

    public function settingsTab()
    {
        if (!current_user_can('manage_options')) {
            _e('You dont have permission to manage options. Please contact site administrator', 'logistra-robots');
            return;
        }

        echo "<div id='logistra-robot-react-settings-app'>Loading...</div>";
        // echo "<div class='logistra-robots-send-order' data-id=12 data-sent=true></div>";
    }

    /* When shipment method is delete, clean up the option. */
    public function deleteTransportRelation($instance_id, $method_id, $id)
    {
        delete_option('logistra-robots-shipping-relation-' . $method_id . ':' . $instance_id);
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {
        // wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/logistra-robots-admin.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {
        $php_to_js_variables = array(
            'wc_ajax_url' => WC()->ajax_url(),
            'security' => wp_create_nonce($this->securityNounce),
            'apikey' => get_option('logistra-robots-apikey', null),
            'token' => get_option('logistra-robots-token', null),
            'wildrobotUsername' => get_option('logistra-robots-wildrobot-username', null),
            'settingsCurrentTab' => get_option('logistra-robots-settings-current-tab', null),
            'senderid' => get_option('logistra-robots-senderid', null),
            'senderName' => get_option('logistra-robots-sender-name', null),
            'woocommercePageTitle' => site_url(),
            'woocommerceShopName' => get_bloginfo("name"),
            // 'transportAgreements' => get_option('logistra-robots-transport-agreements', wp_json_encode(new stdClass())),
            // 'transportAgreementsHash' => get_option('logistra-robots-transport-agreements-hash', null),
            'printerDefault' => get_option('logistra-robots-printer-default', null),
            'printerInterval' => get_option('logistra-robots-printer-interval', null),
            'printerIntervalTime' => get_option('logistra-robots-printer-interval-time', null),
            'completeOrder' => get_option('logistra-robots-setting-complete-order', null),
            'sendConsignmentOnCompleteOrder' => get_option('logistra-robots-setting-send-consignment-on-complete-order', null),
            'printers' => json_encode(get_option('logistra-robots-printers', wp_json_encode(array()))),
            'pickupPointCart' => get_option('logistra-robots-pickuppoint-cart', 'yes'),
            'pickupPointCheckout' => get_option('logistra-robots-pickuppoint-checkout', 'yes'),
            'addOrgNameToOrder' => get_option('logistra-robots-add-org-name-to-order', 'no'),
            'staticWeightOnOrders' => get_option('logistra-robots-static-weight-on-orders', 'no'),
            'staticWeightInKg' => get_option('logistra-robots-static-weight-in-kg', 0),
            'transferMethod' => get_option('logistra-robots-selected-transfer-method', null),
            'transferTime' => get_option('logistra-robots-selected-transfer-time',  null),
            'woocommerceEmail' => get_option('admin_email'),
            'consignmentDescriptionProductNames' => get_option('logistra-robots-consignment-description-product-names', null),
            'returnAddress1' => get_option('logistra-robots-return-address-address1', ""),
            'returnAddress2' => get_option('logistra-robots-return-address-address2', ""),
            'returnCity' => get_option('logistra-robots-return-address-city', ""),
            'returnCountry' => get_option('logistra-robots-return-address-country', ""),
            'returnPostcode' => get_option('logistra-robots-return-address-postcode', ""),
            'useTableRateIntegration' => get_option('logistra-robots-use-table-rate-integration', null),
            'activatedManualPickup' => get_option('logistra-robots-activated-manual-pickup', null),
            'freightTrackUrlEmail' => get_option('logistra-robots-freight-track-url-email', "yes"),
            'freeFreightNotice' => get_option('logistra-robots-free-freight-notice', ""),
            'freeFreightAlmostValue' => get_option('logistra-robots-free-freight-almost-value', 0),
        );


        /* 
            Settings.js
        */
        if (array_key_exists('tab', $_GET) && $_GET["tab"] === "logistra_robots_tab") {
            wp_enqueue_script($this->plugin_name . '-stripe', "https://js.stripe.com/v3/", null, $this->version, false);
        }

        $adminJs = $this->get_hashed_js_dir_url('react', 'admin');
        wp_enqueue_script($this->plugin_name . '-logistra-admin-js', $adminJs, null, $this->version, true);
        wp_localize_script(
            $this->plugin_name . '-logistra-admin-js',
            'logistraRobotsAdmin',
            $php_to_js_variables
        );
    }

    public function get_hashed_js_dir_url($folder, $file)
    {
        if (!empty(glob(plugin_dir_path(__FILE__) . 'partials/' . $folder . '/' . $file . '*.js'))) {
            $full_path = glob(plugin_dir_path(__FILE__) . 'partials/' . $folder . '/' . $file . '*.js')[0];
            $app_pos = strpos($full_path, $file . ".");
            $file_name = substr($full_path, $app_pos);
            return plugin_dir_url(__FILE__) . 'partials/' . $folder . '/' . $file_name;
        } else {
            var_dump("Cannot get WildRobot JS files for => " . $folder . '/' . $file);
        }
    }
}
