<?php

/**
 * Fired during plugin activation
 *
 * @link       https://wildrobot.app/logistra
 * @since      1.0.0
 *
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/includes
 * @author     Robertosnap <robertosnap+spam@pm.me>
 */
class Logistra_Robots_Order
{

    public $order;
    public $error = false;
    public $messages = array();

    public function __construct($orderId)
    {
        $this->order = wc_get_order($orderId);
        // DO i need this
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-helpers.php';
    }

    public function handle_created_consignments($consignment)
    {
        $orderNoteText = "";

        if ($consignment === null) {
            $this->error = true;
            $nullErrorMEssage = __('Ingen kobling til WildRobot server.', 'logistra-robots');
            $orderNoteText .= $nullErrorMEssage . '</br>';
            array_push($this->messages, $nullErrorMEssage);
            return false;
        }

        if (property_exists($consignment, 'error')) {
            $this->error = true;
            $orderNoteText .= __('Det oppsto en feil ved sending av denne ordren til Logistra.', 'logistra-robots') . '</br>';
            if (is_string($consignment->error)) {
                array_push($this->messages, $consignment->error);
                $orderNoteText .= $consignment->error . '</br>';
            } else if (is_array($consignment->error)) {
                foreach ($consignment->error as $error) {
                    array_push($this->messages, $error);
                    $orderNoteText .= $error . '</br>';
                }
                /* Add text to order notes */
                $this->order->add_order_note($orderNoteText);
                return false;
            }
        }

        /* Log order created */
        if (property_exists($consignment, 'id') && property_exists($consignment, 'number')) {
            $consignmentConfirmationText = sprintf(__('Forsendelse opprettet. ' . '</br></br>' . ' Sendingsnummer: %s.', 'logistra-robots') . '</br></br>', $consignment->{'number'});
            $orderNoteText .= $consignmentConfirmationText;
            if (property_exists($consignment, 'product') && property_exists($consignment->product, 'name')) {
                array_push($this->messages, sprintf(__("Transport %s opprettet med %s"), $consignment->{'number'}, $consignment->product->name));
            } else {
                array_push($this->messages, sprintf(__("Transport %s opprettet"), $consignment->{'number'}));
            }
        }

        /* Cost estimate */
        if (property_exists($consignment, 'cost-estimate-gross') &&  $consignment->{'cost-estimate-gross'} != 0) {
            $costEstimateText = sprintf(__('Estimert fraktkostnad:', 'logistra-robots') . '</br>' . __('Kr. %s eks. mva.', 'logistra-robots') . '</br></br>', $consignment->{'cost-estimate-gross'});
            $orderNoteText .= $costEstimateText;
            array_push($this->messages, sprintf(__("Estimert fraktkostnad: %s"), $consignment->{'cost-estimate-gross'}));
        }

        /* tracking-url */
        if (property_exists($consignment, 'tracking-url')) {
            $trackingUrlText   = sprintf('%1$s <a href="%2$s" target="_blank">%2$s</a> ', __('Sporingslink:', 'logistra-robots'), $consignment->{'tracking-url'});
            $orderNoteText .= $trackingUrlText . '</br></br>';
            if (!update_post_meta($this->order->id, 'logistra-robots-tracking-url', $consignment->{'tracking-url'})) {
                // Nothing needed to do if this is set again.
            }
        }

        /* consignment-pdf */
        if (property_exists($consignment, 'consignment-pdf')) {
            $notePdfUrl = sprintf('%1$s <a href="%2$s" target="_blank">%2$s</a> ', __('Etikett:', 'logistra-robots'), $consignment->{'consignment-pdf'});
            $orderNoteText .= $notePdfUrl . '</br></br>';
        }

        /* Log if we have sent this order before */
        if (!update_post_meta($this->order->id, 'logistra-robots-sent', 'yes')) {
            $orderNoteText .= __('Denne ordren har tidligere blitt sendt til Logistra Cargonizer.', 'logistra-robots') . '</br></br>';
        }

        /* Should we complete the order */
        if ((get_option('logistra-robots-setting-complete-order') == 'yes') ? true : false) {
            $status = 'completed';
            // Check if this order status exist
            if (wc_is_order_status('wc-completed')) {
                $this->order->update_status('completed');
                $orderNoteText .= __('Ordren ble fullført ved transportopprettelse.', 'logistra-robots') . '</br></br>';
            }
        }

        /* Add text to order notes */
        $this->order->add_order_note($orderNoteText);
    }

    private function get_order_description()
    {
        if (get_option('logistra-robots-consignment-description-product-names', null) === "yes") {
            $all_products = $this->order->get_items();
            $name_array = array();
            foreach ($all_products as $key => $product) {
                array_push($name_array, $product->get_quantity() . "x" . $product->get_name() . ",");
            }
            $transport_description = join(" ", $name_array);
        } else {
            $transport_description = sprintf(__('%s produkter', 'logistra-robots'), count($this->order->get_items()));
        }
        return $transport_description;
    }

    private function get_consignment_body($transport_agreement, $autoTransfer, $product_id, $packages, $transport_description, $weight)
    {
        return array(
            "consignment" => array(
                (object) array(
                    '_attributes' => array(
                        'transport_agreement' =>  $transport_agreement,
                        'print' => false,
                        'estimate' => true // todo: implement freight cost calculation
                    ),
                    // If no spesific transfer is set and we want to transfer consignments right away we use this 
                    'transfer' => $autoTransfer,
                    'values' => array(
                        'value' => array(
                            // Here we need to pass in multiple value elements. We can do it this way and backend will parse it correctly.
                            (object) array(
                                '_attributes' => array(
                                    'name' => 'ordre_id',
                                    'value' => $this->order->id,
                                )
                            ),
                            (object) array(
                                '_attributes' => array(
                                    'name' => 'provider',
                                    'value' => "WildRobot.app",
                                )
                            ),
                            (object) array(
                                '_attributes' => array(
                                    'name' => 'provider-email',
                                    'value' => "bipbop@wildrobot.app",
                                )
                            )
                        ),
                    ),
                    "booking_request" => get_option('logistra-robots-activated-manual-pickup', null) == "yes" ? true : false,
                    'product' => $product_id,
                    "parts"      => array(
                        "consignee" => array(
                            "number"   => apply_filters(
                                'logistra_robots_parts_cosignee_number',
                                esc_html((string) $this->order->get_customer_id()),
                                $this->order
                            ),
                            "name"     => apply_filters('logistra_robots_parts_cosignee_name', esc_html($this->get_name_of_order($this->order->get_formatted_shipping_full_name())), $this->order),
                            "address1" => esc_html($this->order->get_shipping_address_1()),
                            "country"  => apply_filters('logistra_robots_parts_cosignee_country', esc_html(strlen($this->order->get_shipping_country()) == 0 ? "NO" : $this->order->get_shipping_country()),  $this->order),
                            "postcode" => esc_html($this->order->get_shipping_postcode()),
                            "city"     => esc_html($this->order->get_shipping_city()),
                            "phone"    => esc_html($this->order->get_billing_phone()),
                            "mobile"   => esc_html($this->order->get_billing_phone()),
                            "email"    => esc_html($this->order->get_billing_email()),
                            "contact-person"    => esc_html($this->order->get_formatted_shipping_full_name()),
                            "address2"    => esc_html($this->order->get_shipping_address_2()),
                        ),

                    ),
                    "items"      => array(
                        array(
                            "item" => array(
                                "_attributes" => array(
                                    "amount"      => $packages,
                                    "description" => esc_html(substr($transport_description, 0, 200)),
                                    "type"        => "PK",
                                    "weight"      => $weight, // todo: implement static volume
                                ),
                            )
                        ),
                    ),
                    "references" => array(
                        "consignor" => $this->order->id,
                        "consignee" => "Ordre nr: " . $this->order->id, // Todo .Dersom DHL skal denne ikk fylles ut
                    ),
                    "messages"   => array(
                        "carrier"   => esc_html($this->order->get_customer_note()),
                        "consignee" => 'Avsender: ' . esc_html(get_bloginfo("name")) . ', Mottaker: ' . esc_html($this->order->get_formatted_shipping_full_name()),
                    ),
                ),
            ),
        ); // END consignments array
    }

    private function get_dimensions($args)
    {
        $array = array();
        if ($args['length'] && $args['height'] && $args['width']) {
            $array["length"] = $args['length'];
            $array["height"] = $args['height'];
            $array["width"] = $args['width'];
            $array["volume"] = ($args['length'] * $args['height'] * $args['width']) / 1000;
        }
        return $array;
    }

    private function get_services($servicesCommaDelimtedStrings, $transportAgreement = null)
    {
        // Go through every added services and add them to the consignment.
        $array = array();
        if ($servicesCommaDelimtedStrings == null) {
            return $array;
        }
        foreach ($servicesCommaDelimtedStrings as $key => $serviceName) {
            if ($serviceName != "") {
                if ($transportAgreement) {
                    // Skip our own added services by checking the the transport agreeement has this service
                    if (array_key_exists('services', $transportAgreement) && array_key_exists('service', $transportAgreement['services'])) {
                        foreach ($transportAgreement['services']['service'] as $taService) {
                            if ($taService['identifier'] === $serviceName) {
                                array_push($array, (object) array(
                                    '_attributes' => array(
                                        'id' => $taService['identifier'],
                                    )
                                ));
                            }
                        }
                    }
                } else {
                    // Not used yet
                    array_push($array, (object) array(
                        '_attributes' => array(
                            'id' => $serviceName,
                        )
                    ));
                }
            }
        }
        return $array;
    }
    private function get_service_partner($servicePartner)
    {
        $array = array();
        if ($servicePartner) {
            $array = array(
                "number" => $servicePartner["number"],
                "name" => $servicePartner["name"],
                "address1" =>  array_key_exists('address1', $servicePartner) ? $servicePartner["address1"] : "",
                "postcode" => $servicePartner["postcode"],
                "city" => $servicePartner["city"],
                "country" => $servicePartner["country"],
            );
        }
        return $array;
    }

    private function get_print($printer, $printerInterval, $printTimeString)
    {

        // Calculate local time
        $nowLocal = strToTime(current_time('mysql'));
        switch ($printerInterval) {
            case 'time':
                $printTimeDate = strtotime($printTimeString);
                $printTimeInFuture = $printTimeDate - $nowLocal > 0 ? true : false;
                if ($printTimeInFuture) {
                    $printTime = $printTimeDate;
                } else {
                    $printTime = strToTime(' +1day', $printTimeDate);
                }
                break;
            case 'manual':
                $printTime = strtotime('+59 year');
                break;
            default:
                $printTime = $nowLocal;
                break;
        }
        // Print time must be UTC 
        return array(
            'printerDefault' => $printer,
            'printTime' => $this->localToUTCtime($printTime)/* + $diffUTC, */
        );
    }

    private function get_transfer($transferInterval, $transferTimeString, $printTime, $autoTransfer)
    {
        $nowLocal = strToTime(current_time('mysql'));
        switch ($transferInterval) {
            case 'time':
                $transferTimeDate = strtotime($transferTimeString);
                // $future = $nowLocal - $transferTimeDate ? false : true;
                $transferTimeAfterPrintTime = $transferTimeDate - $printTime > 0 ? true : false;
                if ($transferTimeAfterPrintTime) {
                    $transferTime = $transferTimeDate;
                } else {
                    $transferTime = strToTime(' +1day', $transferTimeDate);
                }
                break;
            case 'manual':
                $transferTime = strtotime('+59 year');
                break;
            case 'none':
                $transferTime = strToTime('+59 year');
                break;
            default:
                $transferTime = $nowLocal;
                break;
        }
        // Transfer time must be UTC 
        return array(
            'transferTime' =>  $this->localToUTCtime($transferTime) /* + $diffUTC */,
            'auto' => $autoTransfer,
        );
    }


    private function get_return_address()
    {
        $array = array();
        // Add return address
        $returnAddress1 = get_option('logistra-robots-return-address-address1', false);
        $returnPostcode = get_option('logistra-robots-return-address-postcode', false);
        $returnCity = get_option('logistra-robots-return-address-city', false);
        $returnCountry = get_option('logistra-robots-return-address-country', false);
        if ($returnAddress1 && $returnAddress1 !== "") {
            if ($returnPostcode && $returnPostcode !== "") {
                if ($returnCity && $returnCity !== "") {
                    if ($returnCountry && $returnCountry !== "") {
                        $array = array(
                            "name"   => apply_filters('logistra_robots_parts_return_address_name', get_bloginfo("name")),
                            "address1"     =>  esc_html(get_option('logistra-robots-return-address-address1', false)),
                            "address2" => esc_html(get_option('logistra-robots-return-address-address2', false)),
                            "postcode"  => esc_html(get_option('logistra-robots-return-address-postcode', false)),
                            "city" => esc_html(get_option('logistra-robots-return-address-city', false)),
                            "country"     => esc_html(get_option('logistra-robots-return-address-country', false)),
                        );
                    }
                }
            }
        }
        return $array;
    }

    public function order_to_consignments($args)
    {
        $args = wp_parse_args($args, array(
            "productId" => null,
            "transportAgreementId" => null,
            "packages" => null,
            "skipServices" => null,
            "weight" => null,
            "length" => null,
            "width" => null,
            "height" => null,
            "services" => null,
            "autoTransfer" => null,
            "servicePartner" => null,
        ));
        $consignments = array();
        $consignmentsArray = array();

        $transport_description = $this->get_order_description();

        $shipping_methods = $this->order->get_shipping_methods();
        $amount_shipping_methods = count($shipping_methods);
        /* 
        Create consignment WITHOUT shipping method
         */
        if ($amount_shipping_methods == 0) {

            // Use passed autoo transfer arg if set, else use default setting
            $autoTransfer = ($args['autoTransfer'] !== null) ? $args['autoTransfer'] : (get_option('logistra-robots-selected-transfer-method', null) === "now") ? true : false;
            if ($args['transportAgreementId'] == null || $args['productId'] == null) {
                $this->error = true;
                array_push($this->messages, "Ingen fraktmetode på ordren, da må du spesifisere transport avtale og transport produkt.");
                return;
            }
            $transport_agreement_id = $args['transportAgreementId'];
            $product_id = $args['productId'];
            $packages = $args['packages'] !== null ? $args['packages'] : 1;
            $weight = $args['weight'] != null ? $args['weight'] : $this->calculate_shipping_weight();
            $consignments = $this->get_consignment_body($transport_agreement_id, $autoTransfer, $product_id, $packages, $transport_description, $weight);

            // Dimensions
            $consignments["consignment"][0]->items[0]["item"]["_attributes"] = array_merge($consignments["consignment"][0]->items[0]["item"]["_attributes"], $this->get_dimensions($args));

            // Return address
            $consignments["consignment"][0]->parts["return_address"] = $this->get_return_address();

            // Print
            $printer = get_option('logistra-robots-printer-default', null);
            $printerInterval = get_option('logistra-robots-printer-interval', null);
            $printTime = get_option('logistra-robots-printer-interval-time', null);
            $consignments["print"] = $this->get_print($printer, $printerInterval, $printTime);

            // transfer
            $transferInterval = get_option('logistra-robots-selected-transfer-method', null);
            $transferTime = get_option('logistra-robots-selected-transfer-time', null);
            $consignments["transfer"] = $this->get_transfer($transferInterval, $transferTime, $printTime, $autoTransfer);

            // Services
            $consignments["consignment"][0]->services["service"] = $this->get_services($args["services"]);

            // Service partner
            $transport_agreement = Logistra_Robots_Helpers::get_transport_agreement_for_shipping_method($product_id);
            $carrier = $transport_agreement['ta_carrier'];
            $selected_pickUpPoint = get_post_meta($this->order->id, '_shipping_service_partner', true);
            $servicePartner = Logistra_Robots_Helpers::get_spesific_or_closest_pickuppoint2($carrier['name'], $this->order->get_shipping_country(), $this->order->get_shipping_postcode(), $args['servicePartner']);
            $consignments["consignment"][0]->parts["service_partner"] = $this->get_service_partner($servicePartner);


            array_push($consignmentsArray, $consignments);
        }
        /* 
        Create consignment WITH shipping method
         */ else {
            foreach ($shipping_methods as $shp) {
                $shippingMethodValues = Logistra_Robots_Helpers::get_shipping_method_values($shp);
                $autoTransfer = ($shippingMethodValues['selectedTransferTime'] === null) && (get_option('logistra-robots-selected-transfer-method', null) === "now") ? true : false;
                $transport_agreement_id = $args['transportAgreementId'] != null ? $args['transportAgreementId'] : $shippingMethodValues["transportAgreementId"];
                $product_id = $args['productId'] != null ? $args['productId'] : $shippingMethodValues["productIdentifier"];
                $packages = $args['packages'] !== null ? $args['packages'] : 1;
                $weight = $args['weight'] != null ? $args['weight'] : $this->calculate_shipping_weight();
                $consignments = $this->get_consignment_body($transport_agreement_id, $autoTransfer, $product_id, $packages, $transport_description, $weight);

                // Dimensions
                $consignments["consignment"][0]->items[0]["item"]["_attributes"] = array_merge($consignments["consignment"][0]->items[0]["item"]["_attributes"], $this->get_dimensions($args));

                // Return address
                $consignments["consignment"][0]->parts["return_address"] = $this->get_return_address();

                // Print
                $shippingMethodSpesificPrinter = ($shippingMethodValues['printer'] !== null && $shippingMethodValues['printer'] !== "");
                $printer = $shippingMethodSpesificPrinter ? $shippingMethodValues['printer'] : get_option('logistra-robots-printer-default', null);
                $shippingMethodSpesificPrintTime = ($shippingMethodValues['selectedPrintTime'] !== null && $shippingMethodValues['selectedPrintTime'] !== "");
                $printerInterval = $shippingMethodSpesificPrintTime ? 'time' : get_option('logistra-robots-printer-interval', null);
                $printTime = $shippingMethodSpesificPrintTime ? $shippingMethodValues['selectedPrintTime'] : get_option('logistra-robots-printer-interval-time', null);
                $consignments["print"] = $this->get_print($printer, $printerInterval, $printTime);

                // transfer
                $shippingMethodSpesificTransferInterval = $shippingMethodValues['selectedTransferTime'] !== null && $shippingMethodValues['selectedTransferTime'] !== "";
                $transferInterval = $shippingMethodSpesificTransferInterval ? $shippingMethodValues['selectedTransferTime'] : get_option('logistra-robots-selected-transfer-method', null);
                $shippingMethodSpesificTransferTime = ($shippingMethodValues['selectedTransferTime'] !== null && $shippingMethodValues['selectedTransferTime'] !== "");
                $transferTime = $shippingMethodSpesificTransferTime ? $shippingMethodValues['selectedTransferTime'] : get_option('logistra-robots-selected-transfer-time', null);
                $consignments["transfer"] = $this->get_transfer($transferInterval, $transferTime, $printTime, $autoTransfer);

                // Services
                if ($args['skipServices'] == null) {
                    $consignments["consignment"][0]->services["service"] = $this->get_services($shippingMethodValues["services"], $shippingMethodValues["transportAgreement"]);
                }

                // Service partner
                $selected_pickUpPoint = get_post_meta($this->order->id, '_shipping_service_partner', true);
                $servicePartner = Logistra_Robots_Helpers::get_spesific_or_closest_pickuppoint($shp, $this->order->get_shipping_country(), $this->order->get_shipping_postcode(), $selected_pickUpPoint);
                $consignments["consignment"][0]->parts["service_partner"] = $this->get_service_partner($servicePartner);

                array_push($consignmentsArray, $consignments);
            }
        }

        $logger = new WC_Logger();
        $context = array('source' => 'logistra-robots');
        $logger->info("=====================Order2Consignment start=======================", $context);
        $logger->info("Loggin order to consignment with id => " . $this->order->id, $context);
        $logger->info(json_encode($consignmentsArray), $context);
        $logger->info("=====================Order2Consignment stop=======================", $context);
        return $consignmentsArray;
    }

    public function get_name_of_order($name)
    {
        // Should we add company information
        if (get_option('logistra-robots-add-org-name-to-order') == 'yes') {
            $org_name_from_order = $this->order->get_shipping_company() ? $this->order->get_shipping_company() : $this->order->get_billing_company();
            if ($org_name_from_order) {
                // removed Name
                $name = $org_name_from_order; /* . " v/" . $name; */
            } else {
                // try to get it from the user
                $org_name_from_customer = (get_user_meta($this->order->get_customer_id(), 'shipping_company', true))
                    ? get_user_meta($this->order->get_customer_id(), 'shipping_company', true)
                    : get_user_meta($this->order->get_customer_id(), 'billing_company', true);
                if ($org_name_from_customer) {
                    $name = $org_name_from_customer; /*  . " v/" . $name; */
                }
            }
            return $name;
        }
        return $name;
    }

    public function localToUTCtime($local)
    {
        $nowLOCAL = strToTime(current_time('mysql'));
        $tz = date_default_timezone_get();
        date_default_timezone_set('UTC');

        $nowUTC = time();

        date_default_timezone_set($tz);
        $diff = $nowUTC - $nowLOCAL;
        $utc = $local + $diff;
        return $utc;
    }

    public function get_closest_pickuppoint($shp, $country, $postcode)
    {
        $shipping = Logistra_Robots_Helpers::get_method_and_instance_from_shipping_method($shp);
        $servicePartnersObject = Logistra_Robots_Backend::get_service_partners(
            $country,
            $postcode,
            Logistra_Robots_Helpers::get_carrier_for_shipment_method(
                $shipping['method_id'],
                $shipping['instance_id']
            )
        );
        return $servicePartnersObject->servicePartners[0];
    }

    public function calculate_shipping_weight()
    {
        $weight    = 0;
        if (sizeof($this->order->get_items(), false) > 0) {
            foreach ($this->order->get_items() as $item) {
                if ($item['product_id'] > 0) {
                    $_product = $this->order->get_product_from_item($item);
                    if (!is_object($_product)) continue; // skip virtual products
                    if (!$_product->is_virtual()) {
                        $product_weight = $_product->get_weight();

                        if ($product_weight) {
                            $weight += $_product->get_weight() * $item['qty'];
                        }
                    }
                }
            }
        }
        if ($weight > 0) $weight = $weight; // todo: implement weight packaging
        return apply_filters('logistra_robots_set_order_weight', round($weight, 3, PHP_ROUND_HALF_DOWN), $this->order->id);
    }

    public function calculate_shipping_volume()
    {
        if (sizeof($this->order->get_items(), false) > 0) {
            $volume = 0;
            foreach ($this->order->get_items() as $item) {
                $_product = $this->order->get_product_from_item($item);
                if (!is_object($_product)) continue; // Skip bugged products
                if (!$_product->is_virtual()) {
                    if ($_product->get_length() && $_product->get_width() && $_product->get_height()) {
                        $length = intval($_product->get_length());
                        $width = intval($_product->get_width());
                        $height = intval($_product->get_height());
                        $product_volume = $length * $width * $height * $item['qty'];
                        if ((intval($product_volume)) > 0) {
                            $volume = $volume + $product_volume;
                        }
                    }
                }
            }
        }
        return apply_filters('logistra_robots_set_order_volume', round($volume, 0, PHP_ROUND_HALF_DOWN), $this->order->id);
    }
}
